import '../src/validators_library_validators.dart';

import '../src/helper.dart';
import '../src/value_exceptions.dart';
import '../src/value_object.dart';

class EmailAddress extends ValueObject<String> {
  /// Default value: `email`
  static String key = 'email';

  /// One of the following [**ValueException**] maybe thrown:
  /// - **RequiredValueException**
  /// - **InvalidValueException**
  factory EmailAddress(String value) {
    if (value.isEmpty) {
      throw const RequiredValueException();
    } else if (isEmail(value)) {
      return EmailAddress._(value);
    }

    throw InvalidValueException(value, message: 'Invalid email address.');
  }

  /// Returns the string **before** the @ character.
  ///
  /// For example: **xy@gmail.com** returns only **xy**
  String get local => _local(toString());

  /// Returns the string **after** the @ character.
  ///
  /// For example: **xy@gmail.com** returns only **gmail.com**
  String get domain => _domain(toString());

  const EmailAddress._(String value) : super(value);

  /// [**map**] can be a `Map`, `String` or `ValueObject`
  ///
  /// One of the following [**ValueException**] maybe thrown:
  /// - **RequiredValueException**
  /// - **InvalidValueException**
  factory EmailAddress.fromJson(dynamic map) {
    if (map is Map) {
      final flatMap = flattenMap(map);
      return EmailAddress(flatMap[key]);
    }
    return EmailAddress(map.toString());
  }

  Map<String, String> toJson() => {key: toString()};
}

String _local(String emailAddress) => emailAddress.split('@')[0];

String _domain(String emailAddress) => emailAddress.split('@')[1];
