import 'dart:math';
import 'dart:convert';

const _hasNumberRegex = '(?=.*[0-9])';
const _hasLowerCharRegex = '(?=.*[a-z])';
const _hasUpperCharRegex = '(?=.*[A-Z])';
const _hasSpecialCharRegex = '(?=.*[ !"#\$%&\'()*+,-./:;<=>?@[\\]^_`{|}~])';

// Phone regex pattern's code taken from here: https://github.com/agungnursatria/regexpattern
/// Phone Number regex
/// Must started by either, "0", "+", "+XX <X between 2 to 4 digit>", "(+XX <X between 2 to 3 digit>)"
/// Can add whitespace separating digit with "+" or "(+XX)"
/// Example: 05555555555, +555 5555555555, (+123) 5555555555, (555) 5555555555, +5555 5555555555
const _phone =
    r'^(0|\+|(\+[0-9]{2,4}|\(\+?[0-9]{2,4}\)) ?)([0-9]*|\d{2,4}-\d{2,4}(-\d{2,4})?)$';

bool _regExpContains(String str, String source) => str.contains(RegExp(source));

bool _regExpMatch(String str, String regex) => RegExp(regex).hasMatch(str);

bool isPhoneNumber(String input) {
  return _regExpMatch(input, _phone);
}

bool hasNumeric(String str) => _regExpContains(str, _hasNumberRegex);

bool hasLowerChar(String str) => _regExpContains(str, _hasLowerCharRegex);

bool hasUpperChar(String str) => _regExpContains(str, _hasUpperCharRegex);

bool hasSpecialChar(String str) => _regExpContains(str, _hasSpecialCharRegex);

String getRandString(int length) {
  assert(length > 3);

  var random = Random.secure();
  var values = List<int>.generate(length, (i) => random.nextInt(255));
  final encodedVal = base64UrlEncode(values);

  return encodedVal.substring(0, encodedVal.length - 2);
}

Map<String, dynamic> flattenMap(dynamic map) {
  final flatMap = <String, dynamic>{};
  map.entries.forEach((e) {
    if (e.value is Map) {
      flatMap.addAll(flattenMap(e.value));
    } else {
      flatMap.addEntries([e]);
    }
  });
  return flatMap;
}
