import '../src/helper.dart';
import '../valid_value_objects.dart';

class FirstName {
  /// Default value: `fistName`
  static String key = 'firstName';

  final Name _name;

  /// [**map**] can be a `Map`, `String` or `ValueObject`
  ///
  /// One of the following [**ValueException**] maybe thrown:
  /// - **RequiredValueException**
  /// - **InvalidValueException**
  factory FirstName.fromJson(dynamic map) {
    if (map is Map) {
      final flatMap = flattenMap(map);
      return FirstName(flatMap[key]);
    }
    return FirstName(map.toString());
  }

  /// One of the following [**ValueException**] maybe thrown:
  /// - **RequiredValueException**
  /// - **InvalidValueException**
  FirstName(String value) : _name = Name(value);

  Map<String, dynamic> toJson() {
    return _name.toJson();
  }

  @override
  String toString() {
    return _name.toString();
  }
}

class LastName {
  /// Default value: `lastName`
  static String key = 'lastName';

  final Name _name;

  /// One of the following [**ValueException**] maybe thrown:
  /// - **RequiredValueException**
  /// - **InvalidValueException**
  factory LastName.fromJson(dynamic map) {
    if (map is Map) {
      final flatMap = flattenMap(map);
      return LastName(flatMap[key]);
    }
    return LastName(map.toString());
  }

  /// One of the following [**ValueException**] maybe thrown:
  /// - **RequiredValueException**
  /// - **InvalidValueException**
  LastName(String value) : _name = Name(value);

  Map<String, dynamic> toJson() {
    return _name.toJson();
  }

  @override
  String toString() {
    return _name.toString();
  }
}

class MiddleName {
  /// Default value: `middleName`
  static String key = 'middleName';
  final Name _name;

  /// One of the following [**ValueException**] maybe thrown:
  /// - **RequiredValueException**
  /// - **InvalidValueException**
  factory MiddleName.fromJson(dynamic map) {
    if (map is Map) {
      final flatMap = flattenMap(map);
      return MiddleName(flatMap[key]);
    }
    return MiddleName(map.toString());
  }

  MiddleName(String value) : _name = Name(value);

  Map<String, dynamic> toJson() {
    return _name.toJson();
  }

  @override
  String toString() {
    return _name.toString();
  }
}

class Name extends ValueObject {
  /// Default value: `name`
  static String key = 'name';

  /// One of the following [**ValueException**] maybe thrown:
  /// - **RequiredValueException**
  /// - **InvalidValueException**
  factory Name(String value) {
    if (value.isEmpty) {
      throw const RequiredValueException();
    } else if (!validName(value)) {
      throw InvalidValueException(value);
    }
    return Name._(value);
  }

  const Name._(String value) : super(value);

  /// One of the following [**ValueException**] maybe thrown:
  /// - **RequiredValueException**
  /// - **InvalidValueException**
  factory Name.fromJson(dynamic map) {
    if (map is Map) {
      final flatMap = flattenMap(map);
      return Name(flatMap[key]);
    }
    return Name(map.toString());
  }

  Map<String, String> toJson() => {key: toString()};
}

bool validName(String input) {
  return input.length > 1;
}
