import 'package:meta/meta.dart';
import '../src/helper.dart';
import '../src/value_exceptions.dart';
import '../src/value_object.dart';

@immutable
class PhoneNumber extends ValueObject<String> {
  /// Default value: `phone`
  static String key = 'phone';

  /// One of the following [**ValueException**] maybe thrown:
  /// - **RequiredValueException**
  /// - **InvalidValueException**
  factory PhoneNumber(String value) {
    if (value.isEmpty) {
      throw const RequiredValueException();
    } else if (isPhoneNumber(value)) {
      return PhoneNumber._(value);
    }
    throw InvalidValueException(value);
  }

  const PhoneNumber._(String value) : super(value);

  /// [**map**] can be a `Map`, `String` or `ValueObject`
  ///
  /// One of the following [**ValueException**] maybe thrown:
  /// - **RequiredValueException**
  /// - **InvalidValueException**
  factory PhoneNumber.fromJson(dynamic map) {
    if (map is Map) {
      final flatMap = flattenMap(map);
      return PhoneNumber(flatMap[key]);
    }
    return PhoneNumber(map.toString());
  }

  Map<String, String> toJson() => {key: toString()};
}
